function [C,t,lowen] = compute_chroma(filename,thres)
% [C,t,lowen] = compute_chroma(filename,thres)
% This function calculates the chromagram from an audio file
%
% Input :
%    * filename : name of the audio file
%    * thres : threshold used to detect silences
% Output :
%    * C : chromagram
%    * t : time vector
%    * lowen : indexes of the detected silences
%
%   This program is used with the kind permission of its authors 
%   Juan Bello (jpbello at nyu dot edu)
%   Christopher Harte (christopher dot harte at elec dot qmul dot ac dot uk)
%
%   Main references :
%   * C. Harte and M. Sandler, Automatic chord identification using a 
%   quantised chromagram, in Proceedings of the Audio Engineering Society, 
%   Barcelona, Spain, 2005.
%   * J. Bello and J. Pickens, A robust mid-level representation for 
%   harmonic content in music signals, in Proceedings of the International
%   Symposium on Music Information Retrieval (ISMIR), London, UK, 2005, 
%   pp. 304311.
%
%   Copyright is owned by its authors.
%
%   This program is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
%   USA


%% Initial parameters

Fsdownsampled=11025/2; % Sampling frequency
Nwindow=4096; % Window size
Nhop=512; % Hop size
if nargin<2
    thres=-55; % Default threshold
end


%% Loading  the audio file and resampling


% Load the audio file
[audiodata,Fs] = wavread(filename);


% Convert audio file to mono
audiodata = mean(audiodata,2);

% Calculate the resampling parameter
downsample = round(Fs/Fsdownsampled);

% Resample
[audiodata,Fsdownsampled] = audiodownsample(audiodata,Fs,downsample);


% Zero padding
pad = zeros((Nwindow/2),1);
ilength = length(audiodata);
audiodata = [pad; audiodata; pad; pad];
    
% Calculate the number of frames
Ntrames = floor(ilength/Nhop) + 1;

% Compute the time vector
t = ((0 : Ntrames - 1).*Nhop)./(Fsdownsampled);
    
    
%% Computing of the CQT

% CQT parameters
binsperoctave=36;
numcqbins=108;
fmin=73.42;
load('sparkernel.mat');

% Compute the STFT
winfft = hanning(Nwindow);
nfft=   size(sparkernel,1);
[fftframes]=stft(audiodata,nfft,winfft,Nhop,Ntrames);


% Compute the CQT
cq = (fftframes' * sparkernel)';

% Compute the chromagram
chroma=circshift(cq2hpcp(cq,Ntrames, numcqbins, binsperoctave),[0 ((binsperoctave/12)+1)/2 - 1]);


%% Tuning algorithm

binshift=chromabias(chroma,binsperoctave); 
chroma=circshift(chroma',[-(binsperoctave/12 * (48-cps2midi(fmin))) - binshift 0]);

    
%% Post-processing     
    
% Median filtering
chroma=medfilt1(chroma,8,[],2);
    
% Detect the silences
E = sum(chroma.^2,1);
lowen = find(10*log10(E+eps)<thres);
if ~isempty(lowen), pos = find(diff(lowen)>1);
    if size(pos)>1, lowen = [lowen(1:min(pos)) lowen(max(pos) + 1: end)]; end;
end

% Compute the tuned chromagram
tmp = reshape(chroma,binsperoctave/12,12,size(chroma,2));
tmp = tmp.*repmat(gausswin(3),[1 12 size(chroma,2)]);
chroma = reshape(sum(tmp,1),12,size(chroma,2));

% Normalize the chromagram
C=chroma/(alpha_norm(alpha_norm(chroma,6.63261),6.63261)+eps);

    
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function binshift=chromabias(chroma,binsperoctave)
% finds bias on peak locations in the chromagram
% binshift is a corrective factor by which the chromagram
% must be circularly shifted to compensate for the bias
res = binsperoctave/12;

if res ~= 1 % this is to avoid shifting 12-dimensional chroma
    % adds last row at the top and first row at the bottom to create
    % circularity - effectively adds 2 to the bpo-length of the vectors:
    new = [chroma(end,:); chroma; chroma(1,:)];
    % pick peaks in the chroma
    pp=[];
    for k=1:size(new,1)
        pp=[pp peakpick(new(k,:))];
    end
    % modularises to res = bpo/12 bins:
    pp = mod(pp,res);
    % corrects the mod value for bin 3
    pp(pp==0)=res;
    % finds the highest concentration of peaks on the bpo/12 bin resolution
    [trash,pos]=max(hist(pp,res));
    % selects the shift
    binshift=pos-((res+1)/2);
else
    binshift = 0;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [hpcpframes] = cq2hpcp(cqframes, Ntrames, numcqbins, binsperoctave)
% CQ2HPCP Convert Constant Q spectrum to a HPCP
%
%   CQ2HPCP(cqframes, info)
%
%   cqframes = vector or matrix constant q spectrum / spectra
%   info = info structure containing parameters
%

%get cq parameters from info structure
num_win = Ntrames;

cqbins = numcqbins;

bpo = binsperoctave;

% calculate the number of octaves we have in the spectrum
num_oct = floor(cqbins/bpo)-1;

hpcpframes(1:num_win, 1:bpo) = 0;



% for each window calculate chromagram
for i = 1 : num_win
    % add each octave of the spectrum into the chromagram
    for oct = 0:num_oct;
        ind = oct*bpo;

        hpcpframes(i, 1:bpo) = hpcpframes(i, 1:bpo) + abs(cqframes(ind+1:ind+bpo,i))';

    end;

    %     % normalise
    %     if max(hpcpframes(i,1:bpo)) > 0
    %         hpcpframes(i,1:bpo) = hpcpframes(i,1:bpo)/max(hpcpframes(i,1:bpo));
    %     else
    %         hpcpframes(i,1:bpo) = 0;
    %     end
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [xdown,newfs] = audiodownsample(xin,fs,fact)
% [xdown,newfs] = audiodownsample(xin,fs,fact)
% downsamples the signal (xin) by a factor (fact)
% fact must be an integer value
%
% by Juan, 05/05/05

if nargin<3, fact = 1; end;

% forces fact to be an integer
fact = floor(fact);
% normalize
xin = xin/(max(xin)+eps);

if fact <= 1
    xdown = xin;
    newfs = fs;
else
    % defines the filter specs,
    [b,a] = ellip(ellipord(0.9/fact, 1/fact, 0.001, 20),0.001,20,0.9/fact);
    % filters the signal
    xin_filt = filter(b,a,xin);
    %% downsample
    xdown = xin_filt(1:fact:end);
    newfs = fs/fact;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function T=peakpick(X)
% T=peakpick(X): return indices of elements of X which are peaks and positive

I=2:(length(X)-1);
T=find((X(I)>X(I-1)) & (X(I)>X(I+1)) & (X(I)>0))+1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function midi=cps2midi(cps)
midi=round(69+12*(log2(cps/440)));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function l=alpha_norm(DF,alpha)
% l=alpha_norm(X,alpha): return alpha-norm norm.
% eg, alpha=2 means normalise
% variance. alpha=1 means normalise abs value. As alpha
% goes large, tends to normalisation by max value.
l=real(mean(abs(DF).^alpha).^(1/alpha));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [S]=stft(x,nfft,w,Nhop,Ntrames)
% This function calculates the STFT of an audio signal
x=x(:);
w=w(:);

Nwindow=length(w);

data=zeros(Nwindow,Ntrames);

for i=1:Ntrames
    data(:,i)=x(1+(i-1)*Nhop:Nwindow+(i-1)*Nhop);
end


data=w(:,ones(1,Ntrames)).*data;

S=conj(fft(data,nfft));












